@@ -38,8 +38,8 @@ MockClientStateMachineInstance* MockClientStateMachineInstance::instance;
 
 class MockConnectCallback : public AsyncSocket::ConnectCallback {
  public:
-  MOCK_METHOD0(_connectSuccess, void());
-  MOCK_METHOD1(_connectErr, void(const AsyncSocketException&));
+  MOCK_METHOD(void, _connectSuccess, ());
+  MOCK_METHOD(void, _connectErr, (const AsyncSocketException&));
 
   void connectSuccess() noexcept override {
     _connectSuccess();
@@ -324,9 +324,10 @@ TEST_F(AsyncFizzClientTest, TestHandshake) {
 
 TEST_F(AsyncFizzClientTest, TestExporterAPI) {
   EXPECT_THROW(
-      client_->getEkm("EXPORTER-Some-Label", nullptr, 32), std::runtime_error);
+      client_->getExportedKeyingMaterial("EXPORTER-Some-Label", nullptr, 32),
+      std::runtime_error);
   completeHandshake();
-  client_->getEkm("EXPORTER-Some-Label", nullptr, 32);
+  client_->getExportedKeyingMaterial("EXPORTER-Some-Label", nullptr, 32);
 }
 
 TEST_F(AsyncFizzClientTest, TestHandshakeError) {
