@@ -1,3 +1,19 @@
+/*
+ * Copyright (c) Facebook, Inc. and its affiliates.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #include <fizz/crypto/aead/OpenSSLEVPCipher.h>
 #include <fizz/experimental/ktls/AsyncKTLSSocket.h>
 #include <fizz/experimental/ktls/KTLS.h>
@@ -153,9 +169,10 @@ class KTLSTest : public ::testing::Test {
 
 class MockTLSCallback : public fizz::AsyncKTLSSocket::TLSCallback {
  public:
-  MOCK_METHOD2(
+  MOCK_METHOD(
+      void,
       receivedNewSessionTicket,
-      void(fizz::AsyncKTLSSocket*, fizz::NewSessionTicket));
+      (fizz::AsyncKTLSSocket*, fizz::NewSessionTicket));
 };
 
 // Tests where we test the read path for AsyncKTLSSocket.
@@ -183,7 +200,7 @@ class KTLSReadTest : public KTLSTest {
     serverSocket->detachNetworkSocket();
 
     serverConn_.reset(new fizz::AsyncKTLSSocket(
-        &evb_, ktlsFDResult.value(), std::move(tlsCb)));
+        &evb_, ktlsFDResult.value(), std::move(tlsCb), nullptr, nullptr));
 
     clientWrite_ = clientToServer.write();
     clientRead_ = serverToClient.read();
@@ -214,7 +231,7 @@ TEST_F(KTLSReadTest, BasicReadWrite) {
         contents->computeChainDataLength(),
         ::send(client_, contents->data(), contents->length(), 0));
 
-    std::array<char, 64> buf;
+    std::array<char, 64> buf = {};
     EXPECT_CALL(mockReadCB_, getReadBuffer(_, _))
         .WillOnce(Invoke([&](void** ret, size_t* size) {
           *ret = buf.data();
@@ -332,6 +349,8 @@ TEST_F(KTLSReadTest, NoTLSCallbackCausesReadErrOnHandshake) {
   conn.reset(new fizz::AsyncKTLSSocket(
       &evb_,
       fizz::KTLSNetworkSocket::unsafeFromExistingKTLSSocket(serverfd),
+      nullptr,
+      nullptr,
       nullptr));
 
   // Client writes a NewSessionTicket to server.
