@@ -176,10 +176,10 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
   /**
    * Get the exported material.
    */
-  virtual Buf getEkm(
+  Buf getExportedKeyingMaterial(
       folly::StringPiece label,
-      const Buf& context,
-      uint16_t length) const = 0;
+      Buf context,
+      uint16_t length) const override = 0;
 
   /**
    * Clean up transport on destruction
