@@ -35,6 +35,18 @@ inline Extension encodeExtension(const ech::ECHIsInner&) {
   return ext;
 }
 
+template <>
+inline Extension encodeExtension(const ech::OuterExtensions& outerExts) {
+  Extension ext;
+  ext.extension_type = ExtensionType::ech_outer_extensions;
+  ext.extension_data = folly::IOBuf::create(0);
+
+  folly::io::Appender appender(ext.extension_data.get(), 20);
+  detail::writeVector<uint8_t>(outerExts.types, appender);
+
+  return ext;
+}
+
 template <>
 inline ech::ClientECH getExtension(folly::io::Cursor& cs) {
   ech::ClientECH clientECH;
@@ -45,4 +57,11 @@ inline ech::ClientECH getExtension(folly::io::Cursor& cs) {
 
   return clientECH;
 }
+
+template <>
+inline ech::OuterExtensions getExtension(folly::io::Cursor& cs) {
+  ech::OuterExtensions outerExts;
+  detail::readVector<uint8_t>(outerExts.types, cs);
+  return outerExts;
+}
 } // namespace fizz
