@@ -19,6 +19,8 @@ namespace ech {
 using Buf = std::unique_ptr<folly::IOBuf>;
 using HpkePublicKey = Buf;
 
+constexpr size_t kEchAcceptConfirmationSize = 8;
+
 enum class ECHVersion : uint16_t {
   Draft9 = 0xfe09,
 };
@@ -29,6 +31,9 @@ struct ECHCipherSuite {
   bool operator==(const ECHCipherSuite& other) const {
     return kdf_id == other.kdf_id && aead_id == other.aead_id;
   }
+  bool operator!=(const ECHCipherSuite& other) const {
+    return kdf_id != other.kdf_id || aead_id != other.aead_id;
+  }
 };
 
 struct ECHConfigContentDraft {
@@ -43,11 +48,30 @@ struct ECHConfigContentDraft {
 struct ECHConfig {
   ECHVersion version;
   Buf ech_config_content;
+
   ECHConfig() {}
   ECHConfig(const ECHConfig& other) {
     version = other.version;
     ech_config_content = other.ech_config_content->clone();
   }
+  ECHConfig(ECHConfig&& other) noexcept {
+    version = other.version;
+    ech_config_content = std::move(other.ech_config_content);
+  }
+  ECHConfig& operator=(const ECHConfig& other) {
+    if (this != &other) {
+      version = other.version;
+      ech_config_content = other.ech_config_content->clone();
+    }
+    return *this;
+  }
+  ECHConfig& operator=(ECHConfig&& other) noexcept {
+    if (this != &other) {
+      version = other.version;
+      ech_config_content = std::move(other.ech_config_content);
+    }
+    return *this;
+  }
 };
 
 } // namespace ech
