@@ -22,8 +22,8 @@
 #include <folly/Overload.h>
 #include <algorithm>
 
-using folly::Future;
 using folly::Optional;
+using folly::SemiFuture;
 
 using namespace fizz::server;
 using namespace fizz::server::detail;
@@ -218,9 +218,13 @@ AsyncActions processEvent(const State& state, Param param) {
     return folly::variant_match(
         actions,
         ::fizz::detail::result_type<AsyncActions>(),
-        [&state](folly::Future<Actions>& futureActions) -> AsyncActions {
+        [&state](SemiFuture<Actions>& futureActions) -> AsyncActions {
+          if (futureActions.isReady()) {
+            // any exception thrown by get will be caught below
+            return std::move(futureActions).get();
+          }
           return std::move(futureActions)
-              .thenError([&state](folly::exception_wrapper ew) {
+              .deferError([&state](folly::exception_wrapper ew) {
                 auto ex = ew.get_exception<FizzException>();
                 if (ex) {
                   return detail::handleError(
@@ -333,16 +337,19 @@ Actions handleInvalidEvent(const State& state, Event event, Param param) {
 }
 
 template <typename T, typename F>
-folly::Future<
+SemiFuture<
     typename folly::futures::detail::valueCallableResult<T, F>::value_type>
 runOnCallerIfComplete(
     folly::Executor* executor,
-    folly::Future<T> future,
+    SemiFuture<T> future,
     F&& func) {
   if (future.isReady()) {
     return func(std::move(future).get());
   } else {
-    return future.via(executor).thenValue(std::forward<F>(func));
+    return std::move(future)
+        .via(executor)
+        .thenValueInline(std::forward<F>(func))
+        .semi();
   }
 }
 
@@ -467,14 +474,15 @@ static Optional<CookieState> getCookieState(
 namespace {
 struct ResumptionStateResult {
   explicit ResumptionStateResult(
-      Future<std::pair<PskType, Optional<ResumptionState>>> futureResStateArg,
+      SemiFuture<std::pair<PskType, Optional<ResumptionState>>>
+          futureResStateArg,
       Optional<PskKeyExchangeMode> pskModeArg = folly::none,
       Optional<uint32_t> obfuscatedAgeArg = folly::none)
       : futureResState(std::move(futureResStateArg)),
         pskMode(std::move(pskModeArg)),
         obfuscatedAge(std::move(obfuscatedAgeArg)) {}
 
-  Future<std::pair<PskType, Optional<ResumptionState>>> futureResState;
+  SemiFuture<std::pair<PskType, Optional<ResumptionState>>> futureResState;
   Optional<PskKeyExchangeMode> pskMode;
   Optional<uint32_t> obfuscatedAge;
 };
@@ -517,7 +525,7 @@ static ResumptionStateResult getResumptionState(
   }
 }
 
-static Future<ReplayCacheResult> getReplayCacheResult(
+static SemiFuture<ReplayCacheResult> getReplayCacheResult(
     const ClientHello& chlo,
     bool zeroRttEnabled,
     ReplayCache* replayCache) {
@@ -525,8 +533,8 @@ static Future<ReplayCacheResult> getReplayCacheResult(
       !getExtension<ClientEarlyData>(chlo.extensions)) {
     return ReplayCacheResult::NotChecked;
   }
-
-  return replayCache->check(folly::range(chlo.random));
+  auto randBuf = folly::IOBuf::copyBuffer(chlo.random, chlo.random.size());
+  return replayCache->check(std::move(randBuf));
 }
 
 static bool validateResumptionState(
@@ -714,15 +722,14 @@ static Buf getHelloRetryRequest(
   return encodedHelloRetryRequest;
 }
 
-static Buf getServerHello(
+static ServerHello getServerHello(
     ProtocolVersion version,
     Random random,
     CipherSuite cipher,
     bool psk,
     Optional<NamedGroup> group,
     Optional<Buf> serverShare,
-    Buf legacySessionId,
-    HandshakeContext& handshakeContext) {
+    Buf legacySessionId) {
   ServerHello serverHello;
 
   serverHello.legacy_version = ProtocolVersion::tls_1_2;
@@ -746,9 +753,7 @@ static Buf getServerHello(
     serverPsk.selected_identity = kPskIndex;
     serverHello.extensions.push_back(encodeExtension(std::move(serverPsk)));
   }
-  auto encodedServerHello = encodeHandshake(std::move(serverHello));
-  handshakeContext.appendToTranscript(encodedServerHello);
-  return encodedServerHello;
+  return serverHello;
 }
 
 static Optional<std::string> negotiateAlpn(
@@ -990,24 +995,24 @@ static Buf getCertificateRequest(
   return encodedCertificateRequest;
 }
 
-static void updateClientHelloIfECH(
-    std::shared_ptr<ech::Decrypter> decrypter,
-    ClientHello& chlo) {
-  if (decrypter) {
-    auto gotChlo = decrypter->decryptClientHello(chlo);
-    if (gotChlo.has_value()) {
-      chlo = std::move(gotChlo.value());
-    }
-  }
-}
-
 AsyncActions
 EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
     handle(const State& state, Param param) {
   ClientHello chlo = std::move(*param.asClientHello());
 
   // Update the client hello if we are using ECH
-  updateClientHelloIfECH(state.context()->getECHDecrypter(), chlo);
+  auto decrypter = state.context()->getECHDecrypter();
+  bool acceptedECH = false;
+  bool requestedECH =
+      findExtension(chlo.extensions, ExtensionType::encrypted_client_hello) !=
+      chlo.extensions.end();
+  if (decrypter) {
+    auto gotChlo = decrypter->decryptClientHello(chlo);
+    if (gotChlo.has_value()) {
+      acceptedECH = true;
+      chlo = std::move(gotChlo->chlo);
+    }
+  }
 
   addHandshakeLogging(state, chlo);
 
@@ -1069,7 +1074,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
       state.context()->getReplayCache());
 
   auto results =
-      collectAllUnsafe(resStateResult.futureResState, replayCacheResultFuture);
+      collectAll(resStateResult.futureResState, replayCacheResultFuture);
 
   using FutureResultType = std::tuple<
       folly::Try<std::pair<PskType, Optional<ResumptionState>>>,
@@ -1083,6 +1088,8 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
        version = *version,
        cipher,
        pskMode = resStateResult.pskMode,
+       acceptedECH,
+       requestedECH,
        obfuscatedAge =
            resStateResult.obfuscatedAge](FutureResultType result) mutable {
         auto& resumption = *std::get<0>(result);
@@ -1238,7 +1245,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
             newReadRecordLayer->setSkipEncryptedRecords(
                 earlyDataType == EarlyDataType::Rejected);
 
-            return Future<Actions>(actions(
+            return SemiFuture<Actions>(actions(
                 MutateState([handshakeContext = std::move(handshakeContext),
                              version,
                              cipher,
@@ -1299,15 +1306,24 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
               AlertDescription::illegal_parameter);
         }
 
-        auto encodedServerHello = getServerHello(
+        auto serverHello = getServerHello(
             version,
             state.context()->getFactory()->makeRandom(),
             cipher,
             resState.has_value(),
             group,
             std::move(serverShare),
-            legacySessionId ? legacySessionId->clone() : nullptr,
-            *handshakeContext);
+            legacySessionId ? legacySessionId->clone() : nullptr);
+
+        if (acceptedECH) {
+          // If accepted, go ahead and set the last random bytes to the
+          // accept_confirmation
+          ech::setAcceptConfirmation(
+              serverHello, handshakeContext->clone(), scheduler);
+        }
+
+        auto encodedServerHello = encodeHandshake(std::move(serverHello));
+        handshakeContext->appendToTranscript(encodedServerHello);
 
         // Derive handshake keys.
         auto handshakeWriteRecordLayer =
@@ -1369,7 +1385,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
          * transcript.
          */
         Optional<Buf> encodedCertificate;
-        Future<Optional<Buf>> signature = folly::none;
+        SemiFuture<Optional<Buf>> signature = folly::none;
         Optional<SignatureScheme> sigScheme;
         Optional<std::shared_ptr<const Cert>> serverCert;
         std::shared_ptr<const Cert> clientCert;
@@ -1389,12 +1405,13 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
             signature = asyncSelfCert->signFuture(
                 *sigScheme,
                 CertificateVerifyContext::Server,
-                toBeSigned->coalesce());
+                std::move(toBeSigned));
           } else {
-            signature = originalSelfCert->sign(
-                *sigScheme,
-                CertificateVerifyContext::Server,
-                toBeSigned->coalesce());
+            signature =
+                folly::makeSemiFuture<Optional<Buf>>(originalSelfCert->sign(
+                    *sigScheme,
+                    CertificateVerifyContext::Server,
+                    toBeSigned->coalesce()));
           }
           serverCert = std::move(originalSelfCert);
         } else {
@@ -1403,6 +1420,11 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
         }
 
         auto clientRandom = std::move(chlo.random);
+
+        ECHStatus echStatus = ECHStatus::NotRequested;
+        if (requestedECH) {
+          echStatus = acceptedECH ? ECHStatus::Accepted : ECHStatus::Rejected;
+        }
         return runOnCallerIfComplete(
             state.executor(),
             std::move(signature),
@@ -1412,6 +1434,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
              cipher,
              clientRandom = std::move(clientRandom),
              group,
+             echStatus,
              encodedServerHello = std::move(encodedServerHello),
              handshakeWriteRecordLayer = std::move(handshakeWriteRecordLayer),
              handshakeWriteSecret = std::move(handshakeWriteSecret),
@@ -1558,6 +1581,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
                    clockSkew,
                    appToken = std::move(appToken),
                    serverCertCompAlgo,
+                   echStatus,
                    clientRandom = std::move(clientRandom),
                    handshakeTime =
                        std::move(handshakeTime)](State& newState) mutable {
@@ -1585,6 +1609,7 @@ EventHandler<ServerTypes, StateEnum::ExpectingClientHello, Event::ClientHello>::
                     newState.serverCertCompAlgo() = serverCertCompAlgo;
                     newState.handshakeTime() = std::move(handshakeTime);
                     newState.clientRandom() = std::move(clientRandom);
+                    newState.echStatus() = echStatus;
                   });
 
               if (earlyDataType == EarlyDataType::Accepted) {
@@ -1742,7 +1767,7 @@ static WriteToSocket writeNewSessionTicket(
   return nstWrite;
 }
 
-static Future<Optional<WriteToSocket>> generateTicket(
+static SemiFuture<Optional<WriteToSocket>> generateTicket(
     const State& state,
     const std::vector<uint8_t>& resumptionMasterSecret,
     Buf appToken = nullptr) {
