@@ -312,8 +312,10 @@ TEST_F(HandshakeTest, ResumeChangeCipher) {
 TEST_F(HandshakeTest, TestEkmSame) {
   expectSuccess();
   doHandshake();
-  auto clientEkm = client_->getEkm("EXPORTER-Some-Label", nullptr, 32);
-  auto serverEkm = server_->getEkm("EXPORTER-Some-Label", nullptr, 32);
+  auto clientEkm =
+      client_->getExportedKeyingMaterial("EXPORTER-Some-Label", nullptr, 32);
+  auto serverEkm =
+      server_->getExportedKeyingMaterial("EXPORTER-Some-Label", nullptr, 32);
   EXPECT_TRUE(IOBufEqualTo()(clientEkm, serverEkm));
   EXPECT_THROW(
       client_->getEarlyEkm("EXPORTER-Some-Label", nullptr, 32), std::exception);
@@ -874,7 +876,7 @@ class MockAsyncFizzClient : public AsyncFizzClient {
       const std::shared_ptr<ClientExtensions>& extensions)
       : AsyncFizzClient(std::move(socket), std::move(fizzContext), extensions) {
   }
-  MOCK_CONST_METHOD0(connecting, bool());
+  MOCK_METHOD(bool, connecting, (), (const));
 };
 
 TEST_F(HandshakeTest, TestFailureOnInvalidCloseNotify) {
@@ -915,7 +917,7 @@ TEST_F(HandshakeTest, TestFailureOnInvalidCloseNotify) {
   EXPECT_TRUE(serverKCB.isCalled());
 }
 
-INSTANTIATE_TEST_CASE_P(
+INSTANTIATE_TEST_SUITE_P(
     SignatureSchemes,
     SigSchemeTest,
     ::testing::Values(
